// Select the needed modules
import { 
  select, 
  csv, 
  scaleLinear,   
  scaleTime,
  axisLeft,
  extent,
  axisBottom  
} from 'd3';


// Capture in a variable the DOM component 'svg'
const svg = select('svg');

// Capture from the attributes 'height' and 'width' from the 
// DOM component 'svg'
// Use (+) to parse string type to int type.
const height = +svg.attr('height');
const width = +svg.attr('width');

//   d.mpg = +d.mpg;
//   d.cylinders = d.cylinders;
//   d.displacement = d.displacement;
//   d.horsepower = d.horsepower;
//   d.weight = d.weight;
//   d.acceleration = d.acceleration;
//   d.year =  d.year;

// Create 'promise' lamda with rendering instructions
// pass as parameter 'data' array, captured from using 'csv'
const render = data => {
  
  const chartTitle = 'A week in San Francisco';
  // Split data for X axis and Y axis
  const xValue = d => d.timestamp;
  const xAxisLabel = 'Time';
  
  const yValue = d => d.temperature;  
  const circleRadius = 6;  
  const yAxisLabel = 'Temperature';
  
  // Set Margins following D3 guidelines
  const margin = {top: 50, right: 40, bottom: 60, left: 90}
  
  // Calculating width and height form the chart 'canvas'
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  // Define X scale as time scale and 
  // .domain with extent function which return the min and max values of the dataset
  // .range from the begining 0 to the eng at innerWidth
  // .nice close the frame with a 'nice' touch
  const xScale = scaleTime()
  	.domain(extent(data,xValue))
    .range([0, innerWidth])
  	.nice();    
  
  // Define Y scale as linear scale and 
  // .domain with extent function which return the min and max values of the dataset
  // .range from the begining innerHeight to the eng at 0 (to reverse scale, due to 0 is the starting point)
  // .nice close the frame with a 'nice' touch
  const yScale = scaleLinear()
  	.domain(extent(data,yValue))
    .range([innerHeight, 0])
  	.nice();  
  
  // Crate a group and render all the chart components inside
  // Use attribute transform to translate (line up) chart components
  // to the top/left corner
  const g = svg.append('g')
  	.attr('transform',`translate(${margin.left},${margin.top})`)
    
  // Get the buttom axis, formates and define the size
  const xAxis = axisBottom(xScale)  
  	.tickSize(-innerHeight)
  	.tickPadding(8);
  
  // Remove tick and lines from the Y scale axis
  // using DOM class references to '.domain' and '.tick lines'
  const yAxis = axisLeft(yScale).tickSize(-innerWidth)
  		.tickPadding(8);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove()
  
  yAxisG.append('text')
  		.attr('class', 'axis-label')
  		.attr('y', -40)
  		.attr('x', -innerHeight / 2)
  		.attr('fill', 'black')
  		.attr('transform','rotate(-90)')
  		.attr('text-anchor','middle')
  		.text(yAxisLabel);
   
  
  // Adding the xAxis to a group, that way I can remove the domain line
  const xAxisG = g.append('g').call(xAxis)
  	.attr('transform',`translate(0,${innerHeight})`);
  
  // Remove the domain axis
  xAxisG.selectAll('.domain').remove();
  
  
  // Add title to X axis, assigning class attribute to use a reference in CSS
  xAxisG.append('text')
  		.attr('class', 'axis-label')
  		.attr('y', 45)
  		.attr('x', innerWidth / 2)
  		.attr('fill', 'black')
  		.text(xAxisLabel);
  
  
  g.append('path')
  	.attr('d',)
  
  // Add bars and passing axis and data values promises as parameters
  g.selectAll('circle').data(data)
  	.enter().append('circle')
  		.attr('cy', d => yScale(yValue(d)))
  		.attr('cx', d => xScale(xValue(d)))
  	  .attr('r',circleRadius);  
  
  // Add graph title
  g.append('text')
  	.attr('class', 'title')
  	.attr('y', -10)
  	.text(chartTitle);
};

csv('https://vizhub.com/curran/datasets/temperature-in-san-francisco.csv').then(data => {
 data.forEach(d => {
   d.temperature = +d.temperature;
   d.timestamp = new Date(d.timestamp);   
 }); 
 render(data);
});